@@ -12,13 +12,15 @@ our @ISA = qw(Exporter);
 
 our %EXPORT_TAGS = ( 'all' => [ qw(
 	num2cardinal
+	case_endings
+	spelled_out_number
 ) ] );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
 our @EXPORT = qw( );
 
-our $VERSION = '0.05';
+our $VERSION = '0.07';
 
 
 ########################
@@ -31,12 +33,7 @@ our $VERSION = '0.05';
 #~~~~~~~~~~~
 # HASH keys:
 #~~~~~~~~~~~
-# 1. 'gender' key -- род:
-#	(default)	m = masculine (Мужской)
-#					f = feminine  (Женский)
-#					n = neuter    (Средний)
-
-# 2. 'case' key -- падеж:          числительное: Количественное     | Порядковое
+# 1. 'case' key -- падеж:          числительное: Количественное     | Порядковое
 #	(default)	n = nominative    - Именительный: есть кто? что?     | какой?
 #					g = genitive      - Родительный:  нет кого? чего?    | какого?
 #					d = dative        - Дательный:    рад кому? чему?    | какому?
@@ -44,6 +41,11 @@ our $VERSION = '0.05';
 #					i = instrumental  - Творительный: оплачу кем? чем?   | каким?
 #					p = prepositional - Предложный:   думаю о ком? о чём?| о каком?
 
+# 2. 'gender' key -- род:
+#	(default)	m = masculine (Мужской)
+#					f = feminine  (Женский)
+#					n = neuter    (Средний)
+
 # 3. 'multi' key  -- единственное(undef|0|'singular') или множественное число (>0|'plural'). By default, 'singular'
 # 4. 'object' key -- inanimate(неодушевлённый) или animate(одушевлённый) предмет. By default, 'inanimate'
 # 5. 'prolog' key -- Preposition (prologue) of numeral
@@ -51,9 +53,49 @@ our $VERSION = '0.05';
 # 7. 'alt' key
 # 8. 'ucfirst' key
 
-sub num2cardinal {
+sub case_endings {
+	$! = 22, return unless defined wantarray;
 	my( $number, %cfg ) = @_;
-	$number =~s/\D+//g;
+	return unless defined $number;
+
+	my( $i, $ns, @s ) = &num2cardinal( $number, %cfg );
+
+	splice @s, $i, $ns - $i + 1, $number;
+
+	return join(' ', @s);
+}
+
+
+sub spelled_out_number {
+	$! = 22, return unless defined wantarray;
+	my( $number, %cfg ) = @_;
+	return unless defined $number;
+
+	my( $i, $ns, @s ) = &num2cardinal( $number, %cfg );
+
+	$s[$i] = '('.$s[$i];
+	$s[$ns] = $s[$ns].')';
+	splice @s, $i, 0, $number;
+
+	return join(' ', @s);
+}
+
+
+sub _num_check {
+	for( $_[0] ) {
+		s/\D+//g;
+		s/^0+/0/;
+		s/^0(?!0)//;
+	}
+}
+
+
+sub num2cardinal {
+	$! = 22, return unless defined wantarray;
+	my( $number, %cfg ) = @_;
+	return unless defined $number;
+
+	&_num_check( $number );
 
 	# extreme index for:
 	use constant VXN  => 5;	# numbers from 5 to <1000
@@ -87,25 +129,26 @@ sub num2cardinal {
 		 my @s = ( $zero );
 
 		# Add the prolog to numeral
-		my $i = &_add_prolog( $prolog, \@s );
+		my $ii = &_add_prolog( $prolog, \@s );
+		my $ns = $#s;
 
 		# Add the epilog to numeral
-		if( $epilog && exists( $epilog->{'root'} ) ) {
+		if( exists $epilog->{'root'} ) {
 			my $j = 0;
 			for my $root ( @{ $epilog->{'root'} } ) {
 				push @s, $root.$epilog->{'ends'}[ $j++ ][0];
 			}
 		}
 
-		$s[$i] = ucfirst $s[$i] if exists $cfg{'ucfirst'};
+		$s[$ii] = ucfirst $s[$ii] if exists $cfg{'ucfirst'};
 
-		return join ' ', @s;
+		return wantarray ? ( $ii, $ns, @s )  : join(' ', @s);
 	}
 
 	return "$number > 999_999_999_999_999 !" if length( $number ) > 15;
 
 	# To get cardinal form DB (БД словоформ числительных)
-	my( $bsw, $plural, $Power ) = &_cardinal_form_db( $case, $multi, $object, $gender );
+	my( $bsw, $plural, $Power ) = &_cardinal_form_db( $case, $multi, $object, $gender, \%cfg );
 
 	my @Dcml = @{ $bsw->{'dcml'} };
 	my @Cent = @{ $bsw->{'cent'} };
@@ -113,23 +156,23 @@ sub num2cardinal {
 	my @tmEnd;	# for ends --- для окончаний 'тысяч','миллион','миллиард','триллион'
 	my @words;	# for ends --- для искомой структуры слов числительных
 	my @s;	# Resulting string --- Результирующая строка
+	my $ns;	# size of numeral string
 
-	while( length $number ) {
-
+	while( length( $number ) && $number ) {
 		if( $number =~/^.$/ ) { # 0..9
-			last unless $number;
 
 			push @s, ( $multi =~/^plural/i && $number == 1 ) ?
 				$plural->{1} :
 				$bsw->{ $gender }{ ($case =~/^a/i && $object =~/^animate/i) ? 'animate' : 'unit'}[ $number ];
 
 			# Add epilog to numeral
-			if( $epilog && exists( $epilog->{'root'} ) ) {
+			if( exists $epilog->{'root'} ) {
 
 				my $j = 0;
 				for my $root ( @{ $epilog->{'root'} } ) {
 					push @s, $root.$epilog->{'ends'}[ $j++ ][ $number < 5 ? $number : VXN ];
 				}
+				$ns = $j;
 
 				$epilog = {};	# epilog END
 			}
@@ -153,25 +196,20 @@ sub num2cardinal {
 		if( length( $number )%3 == 0 ) { # Сотни: 100 .. 999, 100_xxx .. 999_xxx, 100_xxx_xxx .. 999_xxx_xxx, etc.
 			$number =~s/^\d//;
 			push @s, $Cent[$&] if $&;
-			next;
 		}
-
-		if( length( $number )%3 == 2 ) { # Десятки: 10 .. 99, 10_xxx .. 99_xxx, 10_xxx_xxx .. 99_xxx_xxx, etc.
+		elsif( length( $number )%3 == 2 ) { # Десятки: 10 .. 99, 10_xxx .. 99_xxx, 10_xxx_xxx .. 99_xxx_xxx, etc.
 			if( $number =~/^1/ ) { # 10... 19...
 				$number =~s/^\d\d//;
-				push @s, $words[$&] if $&;
+				push @s, $words[$&];
 
 				push @s, $Power->[$i].$tmEnd[0] if length $Power->[$i];
 			}
-			else { # 20... 99...
+			else { # 00, 20... 99...
 				$number =~s/^\d//;
 				push @s, $Dcml[$&] if $&;
 			}
-
-			next;
 		}
-
-		if( length( $number )%3 == 1 ) { # Единицы: 0..., 1,..., 9...
+		else { # length( $number )%3 == 1  # Единицы: 0..., 1,..., 9...
 			$number =~s/^\d//;
 			my $d = $&;
 			if( $d ) {
@@ -181,7 +219,7 @@ sub num2cardinal {
 			if( $s[-1] !~/^(?:м|трил)/ ) { # ещё не добавлено миллион | миллиард | триллион
 				my $w = ( $multi =~/^plural/i && $d == 1 ) ?
 								$plural->{ $i < 2 ? 'eT' : 'eMT'} :
-								$tmEnd[ $d ];
+								$tmEnd[ $d < 5 ? $d : 0 ];
 
 				push @s, $Power->[$i].$w;
 			}
@@ -191,31 +229,32 @@ sub num2cardinal {
 	}
 
 	# Add prolog to numeral
-	my $i = &_add_prolog( $prolog, \@s );
+	my $ii = &_add_prolog( $prolog, \@s );
+	$ns = ( defined $ns ) ? $#s - $ns : $#s;
 
-	$s[$i] = ucfirst $s[$i] if exists $cfg{'ucfirst'};
+	$s[$ii] = ucfirst $s[$ii] if exists $cfg{'ucfirst'};
 
 	# Add epilog to numeral
-	if( $epilog && exists( $epilog->{'root'} ) ) {
+	if( exists $epilog->{'root'} ) {
 
 		# Choice between
-		my $i = $s[-1] =~/^(?:ты|ми|трил)/ ?
+		my $k = $s[-1] =~/^(?:ты|ми|трил)/ ?
 				TMMT :	# тысяча | миллион | миллиард | триллион
 				VXN;	# other
 
 		my $j = 0;
 		for my $root ( @{ $epilog->{'root'} } ) {
-			push @s, $root.$epilog->{'ends'}[ $j++ ][ $i ];
+			push @s, $root.$epilog->{'ends'}[ $j++ ][ $k ];
 		}
 
 	}
 
-	return join ' ', @s;
+	return wantarray ? ( $ii, $ns, @s )  : join(' ', @s);
 }
 
 
 sub _cardinal_form_db {
-	my( $case, $multi, $object, $gender ) = @_;
+	my( $case, $multi, $object, $gender, $cfg ) = @_;
 
 	# База словоформ числительных (singular -- единственно число, default)
 	my %bsw = (
@@ -243,10 +282,10 @@ sub _cardinal_form_db {
 				'','сто','двести','триста','четыреста','пятьсот','шестьсот','семьсот','восемьсот','девятьсот'
 			],
 			'eT' => [ # окончания для 'тысяч' (0й - для 10..19; десятков: 10,20,...,90 и сотен: 100,200,...,900)
-				'','а', ('и') x 3, ('') x 5
+				'','а', ('и') x 3 # , ('') x 5
 			],
 			'eMT' => [ # окончания для 'миллион','миллиард','триллион' (0й - для 10..19; десятков: 10,20,...,90 и сотен: 100,200,...,900)
-				'ов','', ('а') x 3, ('ов') x 5
+				'ов','', ('а') x 3 # , ('ов') x 5
 			],
 		}, #--------------------------------------------
 		'g' => { # genitive (Родительный падеж): кого? чего?
@@ -273,10 +312,10 @@ sub _cardinal_form_db {
 				'','ста','двухсот','трёхсот','четырёхсот','пятисот','шестисот','семисот','восьмисот','девятисот'
 			],
 			'eT' => [ # окончания для 'тысяч'
-				'','и', ('') x 8
+				'','и', ('') x 3 # 8
 			],
 			'eMT' => [ # окончания для 'миллион','миллиард','триллион'
-				'ов','а', ('ов') x 8
+				'ов','а', ('ов') x 3 # 8
 			],
 		}, #--------------------------------------------
 		'd' => { # dative (Дательный падеж): кому? чему?
@@ -303,10 +342,10 @@ sub _cardinal_form_db {
 				'','ста','двумстам','трёхстам','четырёхстам','пятистам','шестистам','семистам','восьмистам','девятистам'
 			],
 			'eT' => [ # окончания для 'тысяч'
-				'ам','е', ('ам') x 8
+				'ам','е', ('ам') x 3 # 8
 			],
 			'eMT' => [ # окончания для 'миллион','миллиард','триллион'
-				'ам','у', ('ам') x 8
+				'ам','у', ('ам') x 3 # 8
 			],
 		}, #--------------------------------------------
 		'a' => { # accusative (Винительный падеж): animate (одушевлённый объект): кого? | inanimate (неодушевлённый объект): что?
@@ -343,16 +382,16 @@ sub _cardinal_form_db {
 				'','сто','двести','триста','четыреста','пятьсот','шестьсот','семьсот','восемьсот','девятьсот'
 			],
 			'eT' => [ # окончания для 'тысяч' (0й - для 10..19 и десятков: 10,20,...,90)
-				'','у', ('и') x 3, ('') x 5
+				'','у', ('и') x 3 # , ('') x 5
 			],
 			'eMT' => [ # окончания для 'миллион','миллиард','триллион' (0й - для 10..19 и десятков: 10,20,...,90)
-				'ов','', ('а') x 3, ('ов') x 5
+				'ов','', ('а') x 3 # , ('ов') x 5
 			],
 		}, #--------------------------------------------
 		'i' => { # instrumental (Творительный падеж) : кем? чем?
 			'masculine' => { # Мужской род, default
 				'unit' => [ # 0..19
-					'','одним','двумя','тремя','четырьмя','пятью','шестью','семью','восьмью','девятью','десятью',
+					'','одним','двумя','тремя','четырьмя','пятью','шестью','семью','восемью','девятью','десятью', # or 'восьмью'  (see p.22)
 'одиннадцатью','двенадцатью','тринадцатью','четырнадцатью','пятнадцатью','шестнадцатью','семнадцатью','восемнадцатью','девятнадцатью'
 				],
 			},
@@ -360,23 +399,31 @@ sub _cardinal_form_db {
 				'unit' => [ # 0..19
 					'','одной' # остальные как для Мужского рода
 				],
-				'alternative' => [ # 0..19
-					'','одною' # остальные как для Мужского рода
-				],
+				'if1' => 'одною', # альтернативная форма
 			},
 			'dcml' => [ # 20, 30,...,90
-				'','','двадцатью','тридцатью','сорока','пятьюдесятью','шестьюдесятью','семьюдесятью','восьмьюдесятью','девяноста'
+'','','двадцатью','тридцатью','сорока','пятьюдесятью','шестьюдесятью','семьюдесятью','восемьюдесятью','девяноста' # or 'восьмьюдесятью' (see p.22)
 			],
 			'cent' => [ # 100..900
-				'','ста','двумястами','тремястами','четырьмястами','пятьюстами','шестьюстами','семьюстами','восьмьюстами','девятьюстами'
+'','ста','двумястами','тремястами','четырьмястами','пятьюстами','шестьюстами','семьюстами','восемьюстами','девятьюстами' # or 'восьмьюстами' (see p.22)
 			],
 			'eT' => [ # окончания для 'тысяч'
-				'ами','ей', ('ами') x 8
-#				'ами','ью', ('ами') x 8
+				'ами','ей', ('ами') x 3
+#				'ами','ью', ('ами') x 3 # разговорная форма (see 'i.T'=>'C')
+#				'ами','ею', ('ами') x 3 # устаревшая форма (see 'i.T'=>'O')
 			],
+			'i.T' => { # формы окончания для 1 'тысяч'
+				'C' => 'ью', # разговорная (colloquial form)
+				'O' => 'ею', # устаревшая (obsolete form)
+			},
 			'eMT' => [ # окончания для 'миллион','миллиард','триллион'
-				'ами','ом', ('ами') x 8
+				'ами','ом', ('ами') x 3 # 8
 			],
+			'i.8' => {	# разговорные формы (see p.22)
+				'unit' => 'восьмью',
+				'dcml' => 'восьмьюдесятью',
+				'cent' => 'восьмьюстами',
+			},
 		}, #--------------------------------------------
 		'p' => { # prepositional (Предложный падеж) : о ком? о чём?
 			'masculine' => { # Мужской род, default
@@ -397,10 +444,10 @@ sub _cardinal_form_db {
 				'','ста','двухстах','трёхстах','четырёхстах','пятистах','шестистах','семистах','восьмистах','девятистах'
 			],
 			'eT' => [ # окончания для 'тысяч'
-				'ах','е', ('ах') x 8
+				'ах','е', ('ах') x 3 # 8
 			],
 			'eMT' => [ # окончания для 'миллион','миллиард','триллион'
-				'ах','е', ('ах') x 8
+				'ах','е', ('ах') x 3 # 8
 			],
 		},
 	);
@@ -449,8 +496,35 @@ sub _cardinal_form_db {
 			$bsw{ $case }{ $g }{'unit'}[$_] //= $bsw{ $case }{'masculine'}{'unit'}[$_];
 
 			$bsw{'a'}{ $g }{'animate'}[$_] //= $bsw{'a'}{'masculine'}{'animate'}[$_]
-				if $case =~/^a/i && $object =~/^animate/i;
+				if $case =~/^a/i && $object =~/^animate/i; # for accusative(Винительный падеж) + одушевлённый объект
 		}
+
+	}
+
+	# Настраиваем альтернативные | разговорные | устаревшие формы, если заданы
+	if( $case =~/^i/ and exists $cfg->{'alt'} ) {
+
+		my $k = 'i.T';	# разговорная|устаревшая форма окончания для 'тысяч'
+		if( defined( $cfg->{'alt'}{ $k } ) and $cfg->{'alt'}{ $k } =~/^(C|O)/i ) {
+			$bsw{'i'}{'eT'}[1] = $bsw{'i'}{ $k }{uc $1};
+		}
+
+		$k = 'i.8';
+		if( exists $cfg->{'alt'}{ $k } ) {
+			for('dcml','cent') {	# 'восьмьюдесятью', 'восьмьюстами',
+				$bsw{'i'}{$_}[8] = $bsw{'i'}{ $k }{$_};
+			}
+
+			my %gg;
+			for my $g ('masculine','feminine', $gender ) {
+				next if exists $gg{ $g };
+				$gg{ $g } = undef;
+
+				$bsw{'i'}{ $g }{'unit'}[8] = $bsw{'i'}{ $k }{'unit'};# 'восьмью'
+			}
+		}
+
+		$bsw{'i'}{'feminine'}{'unit'}[1] = $bsw{'i'}{'feminine'}{'if1'} if exists $cfg->{'alt'}{'if1'};
 	}
 
 	# Корректируем окончания для accusative(Винительный падеж) + одушевлённый объект + множественное число
@@ -464,15 +538,19 @@ sub _cardinal_form_db {
 }
 
 
-# Add prolog to numeral, e.g. ['с','со=с',...]
+# Add prolog to numeral, e.g. ['с','со=ст',...]
 sub _add_prolog {
 	my( $prolog, $s ) = @_;
-	return 0 if ! @$prolog or ! @$s or (~~@$prolog < 2 and ( ! defined( $prolog->[0] ) or ! length( $prolog->[0] ) ) );
+	return 0 if ! @$prolog or (~~@$prolog < 2 and ( ! defined( $prolog->[0] ) or ! length( $prolog->[0] ) ) );
 
-	my $p = shift @$prolog;	# get 1st element (e.g. 'с') for any numeral
+	# get element 0 in order (e.g. 'с') for any numeral
+	my $p = ( !defined( $prolog->[0] ) or ! length( $prolog->[0] ) or $prolog->[0] =~/=/ ) ?
+		undef :
+		shift @$prolog;
 
+	# get remaining elements
 	my $i = 0;
-	for( @$prolog ) {	# ['со=с',...]
+	for( @$prolog ) {	# ['со=ст',...]
 		next unless $_;
 
 		my( $k, $m ) = split '=';
@@ -486,7 +564,7 @@ sub _add_prolog {
 		}
 	}
 
-	if( defined( $p ) && length $p ) {
+	if( defined $p ) {
 		unshift @$s, $p;
 		$i = 1;
 	}
@@ -502,24 +580,23 @@ sub _ref_prolog {
 	if( ref($prolog) eq 'ARRAY') {
 		return ~~@$prolog ? [ @$prolog ] : [ ];
 	}
-
-	if( ref($prolog) eq 'HASH') {
+	elsif( ref($prolog) eq 'HASH') {
 		my @p;
 		for my $k (sort keys %$prolog ) {
-			if( defined( $prolog->{$k} ) && length( $prolog->{$k} ) ) {
-				my $v = $prolog->{$k};
-				$k =~s/^\s+|\s+$//g;
+			my $v = $prolog->{$k};
+			$k =~s/^\s+|\s+$//g;
+			next unless length $k or $v;
+
+			if( length $v ) {
 				push @p, "$k=$v";
 			}
 			else {
-				$k =~s/^\s+|\s+$//g;
 				unshift @p, $k;
 			}
 		}
 		return ~~@p ? \@p : [ ];
 	}
-
-	if( ref(\$prolog) eq 'SCALAR') {
+	elsif( ref(\$prolog) eq 'SCALAR') {
 		$prolog =~s/^\s+|\s+$//g;
 
 		# genitive - Родительный: кого? чего?
@@ -555,6 +632,10 @@ sub _ref_prolog {
 			$$case = 'd'; # dative
 			return [ $prolog ];
 		}
+		elsif( $prolog =~ /^благодаря$/i ) {
+			$$case = 'd' if $$case !~/^[d]/;
+			return [ $prolog ];
+		}
 
 		# accusative - Винительный: кого? что?
 		if( $prolog =~ /^(?:
@@ -582,52 +663,46 @@ sub _ref_prolog {
 
 		# genitive (Родительный) || dative (Дательный, by default)
 		if( $prolog =~ /^согласно$/i ) {
-			$$case = 'd' if ! $$case or $$case !~/^[gd]/;
+			$$case = 'd' if $$case !~/^[gd]/;
 			return [ $prolog ];
 		}
 
 		# genitive (Родительный, by default) || instrumental (Творительный)
 		if( $prolog =~ /^(?:про)?меж(?:ду)?$/i ) {
-			$$case = 'g' if ! $$case or $$case !~/^[gi]/;
-			return [ $prolog ];
-		}
-
-		# accusative (Винительный) || dative (Дательный, by default)
-		if( $prolog =~ /^благодаря$/i ) {
-			$$case = 'd' if ! $$case or $$case !~/^[ad]/;
+			$$case = 'g' if $$case !~/^[gi]/;
 			return [ $prolog ];
 		}
 
 		# accusative (Винительный) || instrumental (Творительный, by default)
 		if( $prolog =~ /^(?:за|подо?)$/i ) {
-			$$case = 'i' if ! $$case or $$case !~/^[ai]/;
+			$$case = 'i' if $$case !~/^[ai]/;
 			return [ $prolog ];
 		}
 
 		# accusative (Винительный) || prepositional (Предложный)
 		if( $prolog =~/^[вВB][ОоOo]?$/ ) {
 			$prolog =~tr/BOo/ВОо/;
-			$$case = 'a' if ! $$case or $$case !~/^[ap]/;
+			$$case = 'a' if $$case !~/^[ap]/;
 			return [ $prolog ];
 		}
 		elsif( $prolog =~ /^[oOоО][бБ]?[oOоО]?$/ ) {
-			$$case = 'p' if ! $$case or $$case !~/^[ap]/;
+			$$case = 'p' if $$case !~/^[ap]/;
 			return ['о','об=од'];
 		}
 		elsif( $prolog =~ /^на$/i ) {
-			$$case = 'a' if ! $$case or $$case !~/^[ap]/;
+			$$case = 'a' if $$case !~/^[ap]/;
 			return [ $prolog ];
 		}
 
 		# accusative (Винительный) || genitive (Родительный) || instrumental (Творительный, by default)
 		if( $prolog =~/^[cCсС][ОоOo]?$/ ) {
-			$$case = 'i' if ! $$case or $$case !~/^[agi]/;
+			$$case = 'i' if $$case !~/^[agi]/;
 			return ['с','со=ст'];
 		}
 
 		# accusative (Винительный) || dative (Дательный, by default) || prepositional (Предложный)
 		if( $prolog =~ /^по$/i ) {
-			$$case = 'd' if ! $$case or $$case !~/^[adp]/;
+			$$case = 'd' if $$case !~/^[adp]/;
 			return [ $prolog ];
 		}
 
@@ -644,7 +719,7 @@ sub _ref_epilog {
 	if( ref(\$epilog) eq 'SCALAR') {
 		my %eRef;
 
-		if( $epilog =~/^(?:RUB|643)$/ ) { # Российский рубль
+		if( $epilog =~/^(?:RU[BR]|643|810|₽|(?i:рубль|ruble))$/ ) { # Российский рубль
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -661,7 +736,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^(?:BYR|974)$/) { # Белорусский рубль
+		elsif( $epilog =~/^(?:BY[BRN]|Br|933|974)$/) { # Белорусский рубль
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -678,7 +753,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^(?:rub|\-643|byr|\-974)$/) { # Российская | Белорусская копейка
+		elsif( $epilog =~/^(?:ru[br]|\-643|\-810|by[brn]|\-933|\-974|(?i:копейка|kopek))$/) { # Российская | Белорусская копейка
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'feminine',
@@ -695,7 +770,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^(?:USD|840)$/) {
+		elsif( $epilog =~/^(?:USD|840|(?:US)?\$|(?i:доллар|dollar))$/) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -712,7 +787,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^(?:usd|\-840)$/) {
+		elsif( $epilog =~/^(?:usd|\-840|(?i:цент|cent))$/) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -729,7 +804,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^(?:CNY|156)$/) {
+		elsif( $epilog =~/^(?:CNY|RMB|156|(?i:юань|yuan))$/) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -746,10 +821,24 @@ sub _ref_epilog {
 				);
 
 		}
-#		elsif( $epilog =~/^(?:cny|-156)$/) {
-#			%eRef = ('root'=>'фын', 'ends'=>['ей','ь', ('я') x 3, ('ей') x 5 ] );
-#		}
-		elsif( $epilog =~/^year$/i) {
+		elsif( $epilog =~/^(?:cny|rmb|-156|(?i:фынь))$/) {
+			%eRef = (
+					'object' => 'inanimate',
+					'gender' => 'masculine',
+					'root' => 'фын',
+					'ends' => {
+							'n' => ['ей','ь', ('я') x 3, ('ей') x 2 ],
+							'g' => ['ей','я', ('ей') x 5 ],
+							'd' => ['ям','ю', ('ям') x 5 ],
+							'a' => ['ей','ь', ('я') x 3, ('ей') x 2 ],
+							'i' => ['ей','ем', ('ями') x 4, 'ей' ],
+							'p' => ['ях','е', ('ях') x 4, 'ей' ],
+					},
+					'plural' => {'n'=>'и', 'g'=>'ей', 'd'=>'ям', 'a'=>'и', 'i'=>'ями', 'p'=>'ях'},
+				);
+
+		}
+		elsif( $epilog =~/^(?:year|год|лет)$/i) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -766,7 +855,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^month$/i) {
+		elsif( $epilog =~/^(?:month|месяц)$/i) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -783,7 +872,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^day$/i) {
+		elsif( $epilog =~/^(?:day|день)$/i) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -800,7 +889,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^hour$/i) {
+		elsif( $epilog =~/^(?:hour|час)$/i) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -817,7 +906,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^min\.$/i) {
+		elsif( $epilog =~/^(?:min|мин)/i) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'feminine',
@@ -834,7 +923,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^sec\.$/i) {
+		elsif( $epilog =~/^(?:sec|сек)/i) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'feminine',
@@ -851,7 +940,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^meter$/i) {
+		elsif( $epilog =~/^(?:meter|метр)$/i) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'masculine',
@@ -868,7 +957,7 @@ sub _ref_epilog {
 				);
 
 		}
-		elsif( $epilog =~/^stamp$/i) {
+		elsif( $epilog =~/^(?:stamp|печат)/i) {
 			%eRef = (
 					'object' => 'inanimate',
 					'gender' => 'feminine',
@@ -895,7 +984,7 @@ sub _ref_epilog {
 				if( ref( \$eRef{'plural'}{ $case } ) eq 'SCALAR') {
 					$eRef{'ends'}{ $case }[1] = $eRef{'plural'}{ $case };
 				}
-				elsif( ref( $eRef{'plural'}{ $case } ) eq 'ARRAY') {
+				else { # if( ref( $eRef{'plural'}{ $case } ) eq 'ARRAY') # a lot of words in the epilogue
 					my $i = 0;
 					for( @{ $eRef{'plural'}{ $case } } ) {
 						$eRef{'ends'}{ $case }[ $i++ ][1] = $_;
@@ -909,7 +998,7 @@ sub _ref_epilog {
 				$epilog->{'root'} = [ $eRef{'root'} ];
 				$epilog->{'ends'}[0] = $eRef{'ends'}{ $case };
 			}
-			elsif( ref( $eRef{'root'} ) eq 'ARRAY') {
+			else { # if( ref( $eRef{'root'} ) eq 'ARRAY') # a lot of words in the epilogue
 				$epilog->{'root'} = $eRef{'root'};
 				$epilog->{'ends'} = $eRef{'ends'}{ $case };
 			}
@@ -919,13 +1008,13 @@ sub _ref_epilog {
 		}
 
 	}
-	elsif( ref($epilog) eq 'HASH' && exists( $epilog->{'root'} ) && exists( $epilog->{'ends'} ) ) {
-		if( exists $epilog->{'object'} ) {
+	elsif( ref($epilog) eq 'HASH' and exists $epilog->{'root'} and exists $epilog->{'ends'} ) {
+		if( defined $epilog->{'object'} ) {
 			$$object = $epilog->{'object'}=~/^\s*[a1-9]/i ? 'animate' : 'inanimate';
 		}
 
 		$$gender = @{{'m'=>'masculine', 'f'=>'feminine', 'n'=>'neuter'}}{lc $1}
-			if exists( $epilog->{'gender'} ) && $epilog->{'gender'} =~/^\s*(m|f|n)/i;
+			if defined( $epilog->{'gender'} ) && $epilog->{'gender'} =~/^\s*(m|f|n)/i;
 
 	}
 	else {
@@ -949,7 +1038,11 @@ Lingua::RU::Numeral - Perl extension for generate Russian wording of numerals fr
 
 Activate the module:
 
-  use Lingua::RU::Numeral qw( num2cardinal );
+  use Lingua::RU::Numeral qw( num2cardinal case_endings spelled_out_number );
+
+or
+
+  use Lingua::RU::Numeral qw(:all);
 
 Usage examples:
 
@@ -1145,6 +1238,37 @@ Will print the results:
   об Одиннадцати рублях
 
 
+=item 13.
+
+Usage for correct case endings only (without numeral written as a string):
+
+  my $number = 1000;
+  my %options = ('prolog' => 'за', 'epilog'=>'CNY');
+
+  say case_endings( $number, %options);	# add 'юань' (Chinese yuan), using an alphabetic currency code
+
+or in verbose form:
+
+  my( $i, $ns, @s ) = &num2cardinal( $number, %options );
+  splice @s, $i, $ns - $i + 1, $number;
+  say join(' ', @s);
+
+Will print the results:
+
+  за 1000 юаней
+
+
+=item 14.
+
+Usage for correct case endings only (without numeral written as a string):
+
+  say spelled_out_number( 100_000, case=>'accusative', 'prolog'=>'за', 'epilog'=>'CNY', 'ucfirst'=>1 );
+
+Will print the results:
+
+  за 100000 (Сто тысяч) юаней
+
+
 =back
 
 =head1 LIMITATIONS
@@ -1171,13 +1295,13 @@ This module have reason only for perl 5.10 and higher.
 
 =head1 ABSTRACT
 
-This module provides function that can be used to generate Russian verbiage for the natural numbers (unsigned integer) and 0 (zero).
+This module provides functions that can be used to generate Russian verbiage for the natural numbers (unsigned integer) and 0 (zero).
 The methods implemented in this module, are all focused on knowledge of Russian grammar.
 
 
 =head1 DESCRIPTION
 
-This module provides function that can be used to generate Russian verbiage for the natural numbers (unsigned integer) and 0 (zero).
+This module provides functions that can be used to generate Russian verbiage for the natural numbers (unsigned integer) and 0 (zero).
 The methods implemented in this module, are all focused on knowledge of Russian grammar.
 
 This module makes verbiage in "short scales" (1,000,000,000 is "один миллиард" rather than "одна тысяча миллионов").
@@ -1188,15 +1312,24 @@ L<https://ru.wikipedia.org/wiki/Системы_наименования_чисе
 
 =head1 SUBROUTINES
 
-Lingua::RU::Numeral provides this subroutine:
+Lingua::RU::Numeral provides these subroutines:
 
     num2cardinal( $number [, %facultative_options ] );
-
+    case_endings( $number [, %facultative_options ] );
+    spelled_out_number( $number [, %facultative_options ] );
 
 =head2 num2cardinal( $number [, %facultative_options ] )
 
-Convert a C<$number> (natural numbers or 0) to Russian text (string), using explicitly specified global C<%facultative_options>,
-otherwise - default options. The following can be global C<%facultative_options>:
+Convert a C<$number> (natural numbers or 0) to Russian text (string) made in a particular calling context (list, scalar or void),
+using explicitly specified global C<%facultative_options>, otherwise - default options.
+
+C<num2cardinal()> returns C<($i, $ns, @s)> if the currently executing subroutine expects to return a list value,
+and Russian C<$text> if it is looking for a scalar value.
+Here C<$i> — index of the B<first numeral> word in the array of words C<@s>;
+C<$ns> — index of the B<last numeral> word in C<@s>;
+C<@s> — the array of words that form the returned Russian text (string) in scalar context, e.g. C<join(' ',@s)>.
+
+The following can be global C<%facultative_options>:
 
 =over 3
 
@@ -1345,27 +1478,26 @@ Internal B<reference list of epilogs> contains the following preconfigured phras
 
 =over 6
 
-=item * C<'RUB'|643> — for Russian ruble (I<Российский рубль>);
-C<'rub'|-643> — for Russian kopek (I<Российская копейка>).
+=item * C<RUB|RUR|643|810|₽|рубль|ruble> — for Russian ruble (I<Российский рубль>);
+C<rub|rur|-643|-810|byb|byr|byn|\-933|\-974|копейка|kopek> — for kopek (I<копейка>).
 
-=item * C<'CNY'|156> — Chinese yuan.
+=item * C<CNY|RMB|156|юань|yuan> — Chinese yuan;
+C<cny|rmb|-156|фынь> — for Chinese cent.
 
-=item * C<'USD'|840> — for United States dollar;
-C<'usd'|-840> — for US cent.
+=item * C<USD|840|US$|$|доллар|dollar> — for United States dollar;
+C<usd|-840|цент|cent> — for US cent.
 
-=item * C<'BYR'|974> — for Belarusian ruble (I<Белорусский рубль>);
-C<'byr'|-974> — for kopek (I<копейка>).
+=item * C<BYR|BYB|BYR|BYN|Br|933|974> — for Belarusian ruble (I<Белорусский рубль>).
 
-=item * Time measures:  C<'year'> — for I<год>, I<лет>;
-C<'month'> — I<месяц>;
-C<'day'> — I<день>;
-C<'hour'> — I<час>;
-C<'min.'> — minute (I<минута>);
-C<'sec.'> — second (I<секунда>).
+=item * Time measures: C<'year|год|лет'>;
+C<'month|месяц'>;
+C<'day|день'>;
+C<'hour|час'>;
+C<'min|мин'> — for minute (I<минута>);
+C<'sec|сек'> — for second (I<секунда>).
 
-=item * C<'meter'> — I<метр>.
-
-=item * C<'stamp'> — I<печать>.
+=item * Miscellaneous: C<'meter|метр'>;
+C<'stamp|печать'>.
 
 =back
 
@@ -1414,15 +1546,87 @@ and a trillion (1_000_000_000_000).
 =back
 
 
-=item * C<'alt'> option — REF to HASH of alternative word forms.
-Currently available for zero only (I<ноль>, by default; C<< 'alt' => {0=>1} >>, to obtain I<нуль>).
+=item * C<'alt'> option — REF to HASH of alternative (colloquial, obsolete) word forms.
+Currently available for:
 
-=item * C<'ucfirst'> option — Returns the numeral with the first character capitalized (Upper case).
+=over 6
+
+=item * zero (0) (I<ноль>, by default; C<< 'alt' => {0=>1} >>, to obtain I<нуль>).
+
+=item * C<'i.T'> key — Colloquial or Obsolete form for 'thousands' (I<тысяч>) in C<'instrumental'> cases (I<Творительный падеж>):
+
+  say num2cardinal( 1000, case => 'i', alt => {'i.T'=>'Colloquial'} ); # or 'i.T'=>'C'
+  say num2cardinal( 1000, case => 'i', alt => {'i.T'=>'Obsolete'} ); # or 'i.T'=>'O'
+  say num2cardinal( 1000, case => 'i'); # traditional established form
+
+Will print the results:
+
+  одной тысячью
+  одной тысячею
+  одной тысячей
+
+=item * C<'i.8'> key — сolloquial form for I<'восьмью'>, I<'восьмьюдесятью'>, I<'восьмьюстами'> in C<'instrumental'> cases (I<Творительный падеж>):
+
+  say num2cardinal( $_, case => 'i', alt => {'i.8'=>'TRUE'} ) for 8000, 80, 800;
+
+Will print the results:
+
+  восьмью тысячами
+  восьмьюдесятью
+  восьмьюстами
+
+Instead traditional established forms:
+
+  восемью тысячами
+  восемьюдесятью
+  восемьюстами
+
+=item * C<'if1'> key — alternate form for I<'одною'> in C<'instrumental'> cases (I<Творительный падеж>) and C<'feminine'> gender:
+
+  say num2cardinal( 1000, case => 'i', gender => 'feminine', alt => {'if1'=>'TRUE'} );
+  say num2cardinal( 1000, case => 'i', gender => 'feminine', alt => {'if1'=>'TRUE', 'i.T'=>'O'} ); # alternate and obsolete form
+
+  одною тысячей
+  одною тысячею
+
+=back
+
+=item * C<'ucfirst'> option — returns the numeral with the first character capitalized (Upper case).
 B<WARNING!> does not affect the C<'prolog'>.
 
 =back
 
 
+=head2 case_endings( $number [, %facultative_options ] )
+
+This subroutine returns the C<'epilog'> corresponding to the numeral and case endings.
+
+This subroutine is only intended to match C<'epilog'> case endings.
+It does NOT convert a C<$number> (natural numbers or 0) to Russian text (string, i.e. a numeral).
+Therefore, this subroutine makes sense when at least C<$number> and C<'epilog'> option are given.
+All global C<%facultative_options>, are the same as C<num2cardinal()> subroutine.
+In fact, this subroutine is a wrapper around the C<num2cardinal()> subroutine 
+with an additional parameter C<case_endings>:
+
+  say case_endings( 1000, 'epilog'=>'RUB');
+
+Will print the results:
+
+  1000 рублей
+
+
+=head2 spelled_out_number( $number [, %facultative_options ] )
+
+This subroutine returns the C<$number> that is padded with Russian numeral in parentheses (text string).
+E.g.:
+
+  say spelled_out_number( 1000, case=>'accusative', 'prolog'=>'за', 'epilog'=>'CNY', 'ucfirst'=>1 );
+
+Will print the results:
+
+  за 1000 (Одну тысячу) юаней
+
+
 =head1 EXPORT
 
 Lingua::RU::Numeral exports nothing by default.
@@ -1439,6 +1643,9 @@ and the tag C<all> exports them all:
 
 Lingua::RU::Numeral is known to run under perl 5.10.0 on Linux.
 
+=head1 CAVEATS
+
+The author of this module is not linguist. If you would like to help me correct numeral errors, please send me an email.
 
 =head1 SEE ALSO
 
